@@ -5,8 +5,11 @@ Created on 2020-10-24
 '''
 from lodstorage.sql import SQLDB
 from pathlib import Path
+import sqlite3
 from collections import OrderedDict
 import mailbox
+from email.header import decode_header, make_header
+from ftfy import fix_text
 import re
 import os
 import sys
@@ -37,7 +40,11 @@ class Thunderbird(object):
         else:
             self.db=db
             self.profile=profile
-        self.sqlDB=SQLDB(self.db,check_same_thread=False,timeout=5)
+        try:
+            self.sqlDB=SQLDB(self.db,check_same_thread=False,timeout=5)
+        except sqlite3.OperationalError as soe:
+            print(f"could not open database {self.db}: {soe}")
+            raise soe
         pass
      
     @staticmethod
@@ -86,6 +93,10 @@ class Mail(object):
         self.keySearch=keySearch
         self.rawMsg=None
         self.headers={}
+        self.fromUrl=None
+        self.fromMailTo=None
+        self.toUrl=None
+        self.toMailTo=None
         query="""select m.*,f.* 
 from  messages m join
 folderLocations f on m.folderId=f.id
@@ -95,7 +106,10 @@ where m.headerMessageID==(?)"""
         #folderId=maillookup['folderId']
         if self.debug:
             print (maillookup)
-        if len(maillookup)>=1:
+        if len(maillookup)==0:
+            self.found=False
+        else:
+            self.found=True
             mailInfo=maillookup[0]
             folderURI=mailInfo['folderURI']
             messageKey=int(mailInfo['messageKey'])
@@ -125,7 +139,8 @@ where m.headerMessageID==(?)"""
                 self.mbox.close()
                 if self.msg is not None:
                     for key in self.msg.keys():
-                        self.headers[key]=self.msg.get(key)
+                        #https://stackoverflow.com/a/21715870/1497139
+                        self.headers[key]=str(make_header(decode_header(self.msg.get(key))))
                     self.txtMsg=""
                     self.html=""
                     # https://stackoverflow.com/a/43833186/1497139
@@ -142,33 +157,94 @@ where m.headerMessageID==(?)"""
                         charset = part.get_content_charset()
                         if charset is None:
                             charset='utf-8'
+                        partname=part.get_param('name')
+                        part.filename=self.fixedPartName(partname,f"part{len(self.msgParts)}")
                         if contentType == 'text/plain' or contentType== 'text/html':
                             part_str = part.get_payload(decode=1)
                             rawPart=part_str.decode(charset)
                             if contentType == 'text/plain':
                                 self.txtMsg+=rawPart
-                            if contentType == 'text/html':
+                            elif contentType == 'text/html':
                                 self.html+=rawPart
                         pass
         # sort the headers
         self.headers=OrderedDict(sorted(self.headers.items()))
+        if "From" in self.headers:
+            fromAdr=self.headers["From"]
+            self.fromMailTo=f"mailto:{fromAdr}"
+            self.fromUrl=f"<a href='{self.fromMailTo}'>{fromAdr}</a>"
+        if "To" in self.headers:
+            toAdr=self.headers["To"]
+            self.toMailTo=f"mailto:{toAdr}"
+            self.toUrl=f"<a href='{self.toMailTo}'>{toAdr}</a>"
         pass
     
+    def fixedPartName(self,partname:str,defaultName:str):
+        '''
+        get a fixed version of the partname
+        
+        
+        Args:
+            partname(str): the name of the part
+            defaultName(str): the default name to use
+        '''
+        # avoid TypeError: expected string or bytes-like object
+        if partname:
+            if type(partname) is tuple:
+                encoding,_unknown,partname=partname
+            filename=str(make_header(decode_header(partname)))
+        else:
+            filename=f"part{len(self.msgParts)}"
+        filename=fix_text(filename)
+        return filename
+    
+    def __str__(self):
+        text=f"{self.user}/{self.mailid}"
+        return text
+    
+    def getHeader(self,headerName:str):
+        '''
+        get the header with the given name
+        
+        Args:    
+            headerName(str): the name of the header
+            
+        Returns:
+            str: the header value
+        '''
+        if headerName in self.headers:
+            headerValue=self.headers[headerName]
+        else:
+            headerValue="?"
+        return headerValue
+    
+    def asWikiMarkup(self)->str:
+        '''
+        convert me to wiki markup in Wikison notation
+        
+        Returns:
+            str: a http://wiki.bitplan.com/index.php/WikiSon notation
+        '''
+        wikison=f"""{{{{mail
+|user={self.user}
+|id={self.mailid}
+|from={self.getHeader('From')}
+|to={self.getHeader('To')}
+|subject={self.getHeader('Subject')}
+}}}}"""
+        return wikison
+        
+    
     def partAsFile(self,folder,partIndex):
         '''
         save the given part to a file and return the filename
         '''
         # TODO: check Index 
         part=self.msgParts[partIndex]
-        contentType=part.get_content_type()
-        charset = part.get_content_charset()
-        # FIXME derive filename from part
-        defaultName="test.pdf"
-        filename = part.get_param('name') or defaultName
-        f = open(f"{folder}/{filename}", 'wb')
+        f = open(f"{folder}/{part.filename}", 'wb')
         f.write(part.get_payload(decode=1))
         f.close()
-        return filename
+        return part.filename
      
     
     @staticmethod
